@@ -41,7 +41,7 @@ appropriate action can be taken.
 use 5.00503;
 use strict;
 
-$Mail::DeliveryStatus::BounceParser::VERSION = '1.512';
+$Mail::DeliveryStatus::BounceParser::VERSION = '1.514';
 
 use MIME::Parser;
 use Mail::DeliveryStatus::Report;
@@ -70,7 +70,6 @@ my @Preprocessors = qw(
   p_compuserve
   p_aol_senderblock
   p_novell_groupwise_5_2
-  p_aol_bogus_250
   p_plain_smtp_transcript
   p_xdelivery_status
 );
@@ -87,9 +86,8 @@ report of a transient nonfatal error, or a spam or virus autoresponse, you'll
 still get back a C<$bounce>, but its C<<$bounce->is_bounce()>> will return
 false.
 
-It is possible that some bounces are not really bounces; for example, when
-Hotmail responds with 554 Transaction Failed, that just means hotmail was
-overloaded at the time, so the user actually isn't bouncing.  To include such
+It is possible that some bounces are not really bounces; such as
+anything that apears to have a 2XX status code.  To include such
 non-bounces in the reports, pass the option {report_non_bounces=>1}.
 
 For historical reasons, C<new> is an alias for the C<parse> method.
@@ -150,6 +148,16 @@ sub parse {
 
   my $first_part = _first_non_multi_part($message);
 
+  # Deal with some common C/R systems like TMDA
+  {
+    last unless ($message->head->get("x-delivery-agent")
+     and $message->head->get("X-Delivery-Agent") =~ /^TMDA/);
+    $self->log("looks like a challenge/response autoresponse; ignoring.");
+    $self->{type} = "Challenge / Response system autoreply";
+    $self->{is_bounce} = 0;
+    return $self;
+  }
+
   # we'll deem autoreplies to be usually less than a certain size.
 
   # Some vacation autoreplies are (sigh) multipart/mixed, with an additional
@@ -244,7 +252,7 @@ sub parse {
     my $part_for_maybe_transient;
     $part_for_maybe_transient = $message
       if $message->effective_type eq "text/plain";
-    $part_for_maybe_transient
+    ($part_for_maybe_transient)
       = grep { $_->effective_type eq "text/plain" } $message->parts
         if $message->effective_type =~ /multipart/
            && $message->effective_type ne 'multipart/report';
@@ -329,8 +337,14 @@ sub parse {
 
     foreach my $para (split /\n\n/, $delivery_status_body) {
       my $report = Mail::Header->new([split /\n/, $para]);
-      $report->combine();
-      $report->unfold;
+
+      {
+        # This is to prevent Mail::Header from warning with apparently
+        # reasonable data in place. -- rjbs, 2006-08-07
+        local $^W = 0;
+        $report->combine;
+        $report->unfold;
+      }
 
       # Some MTAs send unsought delivery-status notifications indicating
       # success; others send RFC1892/RFC3464 delivery status notifications
@@ -374,7 +388,22 @@ sub parse {
 
       $report->replace(email      => $email);
       $report->replace(reason     => $reason);
-      $report->replace(std_reason => _std_reason($report->get("diagnostic-code")));
+      if (my $status = $report->get('Status')) {
+        # RFC 1893... prefer Status: if it exists and is something we know
+        # about
+        # Not 100% sure about 5.1.0...
+        if ($status =~ /^5\.1\.[01]$/)  {
+          $report->replace(std_reason => "user_unknown");
+        } elsif ($status eq "5.1.2") {
+          $report->replace(std_reason => "domain_error");
+        } elsif ($status eq "5.2.2") {
+          $report->replace(std_reason => "over_quota");
+        } else {
+          $report->replace(std_reason => _std_reason($report->get("diagnostic-code")));
+        }
+      } else {
+        $report->replace(std_reason => _std_reason($report->get("diagnostic-code")));
+      }
       $report->replace(
         host => ($report->get("diagnostic-code") =~ /\bhost\s+(\S+)/)
       );
@@ -394,11 +423,6 @@ sub parse {
           . "; no_problemo."
         );
 
-        unless ($report->get("host") =~ /\baol\.com$/i) {
-          $report->replace(std_reason => "no_problemo");
-        } else {
-          $self->log("but it's aol telling us that; not going to believe it.");
-        }
       }
 
       unless ($arg->{report_non_bounces}) {
@@ -449,7 +473,7 @@ sub parse {
     #
 
     my @delivery_status_parts = grep {
-      $_->effective_type !~ /rfc822|html/ and not $_->is_multipart
+      $_->effective_type !~ /rfc822|html|image/ and not $_->is_multipart
     } $message->parts;
 
     # $self->log("error parts: @{[ map { $_->bodyhandle->as_string }
@@ -463,20 +487,21 @@ sub parse {
     # they usually say "returned message" somewhere, and we can split on that,
     # above and below.
 
-    if (($message->bodyhandle->as_string||'') =~ $Returned_Message_Below) {
+    my $body_string = $message->bodyhandle->as_string || '';
+
+    if ($body_string =~ $Returned_Message_Below) {
       my ($stuff_before, $stuff_splitted, $stuff_after) =
         split $Returned_Message_Below, $message->bodyhandle->as_string, 3;
       # $self->log("splitting on \"$stuff_splitted\", " . length($stuff_before)
       # . " vs " . length($stuff_after) . " bytes.") if $DEBUG > 3;
       push @{$self->{reports}}, $self->_extract_reports($stuff_before);
       $self->{orig_text} = $stuff_before;
-    } elsif (/(.+)\n\n(.+?Message-ID:.+)/is) {
+    } elsif ($body_string =~ /(.+)\n\n(.+?Message-ID:.+)/is) {
       push @{$self->{reports}}, $self->_extract_reports($1);
       $self->{orig_text} = $2;
     } else {
-      push @{$self->{reports}},
-        $self->_extract_reports($message->bodyhandle->as_string);
-      $self->{orig_text} = $message->bodyhandle->as_string;
+      push @{$self->{reports}}, $self->_extract_reports($body_string);
+      $self->{orig_text} = $body_string;
     }
   }
   return $self;
@@ -520,14 +545,11 @@ sub _extract_reports {
   # blah blah 2
   #
 
-  foreach my $line (split/\n/, $text) {
-    # $self->log("-ext- looking for error in $line") if $DEBUG > 3;
-  }
-
   # we'll break it up accordingly, and first try to detect a reason for email 1
   # in section 1; if there's no reason returned, we'll look in section 0.  and
   # we'll keep going that way for each address.
 
+  return unless $text;
   my @split = split(/(\S+\@\S+)/, $text);
 
   foreach my $i (0 .. $#split) {
@@ -784,24 +806,42 @@ sub _std_reason {
     /try.again.later/is or
     /mailbox\b.*\bfull/ or
     /storage/i          or
-    /quota/i
+    /quota/i            or
+    /\s552\s/           or
+    /\s#?5\.2\.2\s/                                   # rfc 1893
   ) {
     return "over_quota";
   }
 
+  my $user_re =
+   '(mailbox|user|recipient|address(ee)?|customer|account|e-?mail|<?\S+?@\S+?>?)';
+
   if (
-    /unknown/i or
-    /disabled|discontinued/is or
-    /can't\s+open/is or
-    /invalid/is or
-    /permanent/is or
-    /unauthorized/is or
-    /unavailable/is or
-    /not(\s+a)?\s+(found|known|listed|valid|recogni|present|exist|activ|allow)/i or
-    /\bno\s+(email|such|mailbox)/i or
-    /unknown|inactive|suspended|cancel+ed/i or
-    /doesn\'t/is or
-    /5\.1\.1/
+    /\s5\.1\.[01]\s/ or                               # rfc 1893
+    /$user_re\s+ (\S+\s+)? (is\s+)?                   # Generic
+     ( (un|not\s+) known| [dw]oes\s?n[o']?t 
+     ( exist|found ) | disabled ) /ix or
+    /no\s+(such\s+)?$user_re/i or                     # Gmail and other
+    /inactive user/i or                               # Outblaze
+    /unknown local part/i or                          # Exim(?)
+    /user\s+doesn't\s+have\s+a/i or                   # Yahoo!
+    /account\s+has\s+been\s+(disabled|suspended)/i or # Yahoo!
+    /$user_re\s+(suspended|discontinued)/i or         # everyone.net / other?
+    /unknown\s+$user_re/i or                          # Generic
+    /$user_re\s+(is\s+)?(inactive|unavailable)/i or   # Hotmail, others?
+    /((in|not\s+a\s+)?valid|no such)\s$user_re/i or   # Various
+    /$user_re\s+(was\s+)?not\s+found/i or             # AOL, generic
+    /$user_re \s+ (is\s+)? (currently\s+)?            # ATT, generic
+     (suspended|unavailable)/ix or 
+    /address is administratively disabled/i or        # Unknown
+    /no $user_re\s+(here\s+)?by that name/i or        # Unknown
+    /<\S+@\S+> is invalid/i or                        # Unknown
+    /address.*not known here/i or                     # Unknown
+    /recipient\s+(address\s+)?rejected/i or           # Cox, generic
+    /User.*not\s+listed\s+in/i or                     # Domino
+    /account not activated/i or                       # usa.net
+    /not\s+our\s+customer/i or                        # Comcast
+    /doesn't handle mail for that user/i              # mailfoundry
   ) {
     return "user_unknown";
   }
@@ -815,10 +855,6 @@ sub _std_reason {
     /no\s+data\s+record\s+of\s+requested\s+type/i
   ) {
     return "domain_error";
-  } elsif (/hotmail.+transaction\s+failed/is) {
-    # next if it's hotmail and they reject 554 transaction failed; that just
-    # means their system is too loaded.
-    return "no_problemo";
   }
 
   return "unknown";
@@ -1126,28 +1162,21 @@ sub p_aol_senderblock {
   my $self    = shift;
   my $message = shift;
 
-  # From: Mail Delivery Subsystem <MAILER-DAEMON@aol.com>
-  # Date: Sun, 16 Feb 2003 19:40:22 EST
-  # To: <owner-batmail@v2.listbox.com>
-  # Subject: Mail Delivery Problem
-  # Mailer: AIRmail [v90_r2.5]
-  # Message-ID: <200302161944.08TTIXHa07448@omr-m05.mx.aol.com>
-  # Lines: 4
-  #
-  #
-  # Your mail to the following recipients could not be delivered because they are not accepting mail from giltaylor@hawaii.rr.com:
-  #         theetopdog
-  #
-
   return unless ($message->head->get("Mailer")||'') =~ /AirMail/i;
   return unless $message->effective_type eq "text/plain";
-  return unless $message->bodyhandle->as_string =~ /Your mail to the following recipients could not be delivered because they are not accepting mail from/i;
+  return unless $message->bodyhandle->as_string =~ /Your mail to the following recipients could not be delivered because they are not accepting mail/i;
 
   my ($host) = $message->head->get("From") =~ /\@(\S+)>/;
 
   my $rejector;
   my @new_output;
   for (split /\n/, $message->bodyhandle->as_string) {
+
+    # "Sorry luser@example.com. Your mail to the...
+    # Get rid of this so that the module doesn't create a report for
+    # *your* address.
+    s/Sorry \S+?@\S+?\.//g;
+
     if (/because they are not accepting mail from (\S+?):?/i) {
       $rejector = $1;
       push @new_output, $_;
@@ -1233,76 +1262,6 @@ sub p_novell_groupwise_5_2 {
   return $message;
 }
 
-sub p_aol_bogus_250 {
-  my ($self, $message) = @_;
-
-  # the SMTP snapshot shows 550.  the actual Diagnostic-Code shows 250.  what
-  # is going on?
-
-  # pennwomen-la@v2.listbox.com/200209/19/1032468845.1444_1.frodo
-  # ----- Transcript of session follows -----
-  # ... while talking to air-xj03.mail.aol.com.:
-  # >>> RCPT To:<robinbw@aol.com>
-  # <<< 550 MAILBOX NOT FOUND
-  # 550 <robinbw@aol.com>... User unknown
-  #
-  # --QAM07349.1032468790/rly-xj03.mx.aol.com
-  # Content-Type: message/delivery-status
-  #
-  # Reporting-MTA: dns; rly-xj03.mx.aol.com
-  # Arrival-Date: Thu, 19 Sep 2002 16:52:48 -0400 (EDT)
-  #
-  # Final-Recipient: RFC822; robinbw@aol.com
-  # Action: failed
-  # Status: 2.0.0
-  # Remote-MTA: DNS; air-xj03.mail.aol.com
-  # Diagnostic-Code: SMTP; 250 OK
-  # Last-Attempt-Date: Thu, 19 Sep 2002 16:53:10 -0400 (EDT)
-
-  return unless $message->head->get("From") =~ /<MAILER-DAEMON\@aol.com>/i;
-  return unless $message->effective_type eq "multipart/report";
-  my ($plain, $error_part) = $message->parts;
-
-  return unless
-    ($error_part->bodyhandle->as_string =~ /Diagnostic-Code: .*250 OK/);
-
-  my %by_email = $self->_analyze_smtp_transcripts($plain->bodyhandle->as_string);
-
-  my (@new_output, $email);
-  # rewrite the diagnostic code in the delivery-status part.
-  for (split /\n/, $error_part->bodyhandle->as_string) {
-    undef $email if /^$/;
-    $email = _cleanup_email($1) if (/^Final-Recipient: .*\s(\S+)$/);
-
-    if (/^Diagnostic-Code:/
-        and
-        exists $by_email{$email}->{smtp_code}
-    ) {
-      $self->log("cleaning up AOL bogosity: before, $_");
-      push @new_output, _construct_diagnostic_code(\%by_email, $email);
-      $self->log("cleaning up AOL bogosity:  after, $new_output[-1]");
-    }
-    push @new_output, $_ and next;
-  }
-
-  if (my $io = $error_part->open("w")) {
-    $io->print(join "\n", @new_output);
-    $io->close;
-  }
-  return $message;
-}
-
-sub _construct_diagnostic_code {
-  my %by_email = %{shift()};
-  my $email = shift;
-  join (" ",
-    "Diagnostic-Code: X-BounceParser;",
-    ($by_email{$email}->{host} ? "host $by_email{$email}->{host} said:" : ()),
-    ($by_email{$email}->{smtp_code}),
-    (join ", ", @{ $by_email{$email}->{errors} })
-  );
-}
-
 sub p_plain_smtp_transcript {
   my ($self, $message) = (shift, shift);
 
@@ -1493,6 +1452,8 @@ sub _cleanup_email {
     s/.*:SMTP=//;
     s/^\s+//;
     s/\s+$//;
+    # hack to get rid of stuff like "luser@example.com...User"
+    s/\.{3}\S+//;
     }
   return $email;
 }
